import {LabLayoutComponentModel} from "../LabLayoutComponentStore";
import {CanvasRenderer} from "./CanvasRenderer";

export class LayoutEditViewRenderer extends CanvasRenderer {
    constructor() {
        super();
    }

    public deleteSelectedComponent(): void {
        if (!this.selectedComponent) return;
        this.layout.deleteComponent(this.selectedComponent);
        this.setSelectedComponent(undefined);
    }

    protected handleDraw(): void {
        // Use the view model to render the positions
        this.layout.current_ordered_component.forEach(c => {
            switch (c.component_type) {
                case "machine_status_dot":
                    this.drawDetailedMachineDot(c);
                    break;
                case "image":
                    this.drawImage(c);
                    break;
                case "alert_group":
                    this.drawDetailedAlertGroup(c);
                    break;
            }
        });
        this.drawSelectedComponentBoundaries();
    }

    protected afterSetLayout(): void {
        // Nothing to do here in edit mode
    }

    private drawDetailedMachineDot(c: LabLayoutComponentModel): void {
        this.drawCircle({x: c.x, y: c.y}, c.width, {fillColor: LayoutEditViewRenderer.MACHINE_STATUS_DOT_COLOR});
        this.drawText(c.name, {x: c.position.x, y: c.position.y - 5})
    }

    private drawDetailedAlertGroup(c: LabLayoutComponentModel): void {
        this.drawAlertGroup(c, {fillColor: CanvasRenderer.COLOR_ALERT_OFF});
        this.drawText(c.name, {x: c.position.x, y: c.position.y - 5})
    }

    private drawSelectedComponentBoundaries(): void {
        if (!this.selectedComponent) return;
        const c = this.selectedComponent;
        this.drawRectangle(c.position, c.width, c.height, LayoutEditViewRenderer.SELECTED_COMPONENT_BORDER_STYLE);
        this.drawLine({x: c.x + c.width - 15, y: c.y + c.height}, {
            x: c.x + c.width,
            y: c.y + c.height - 15
        }, LayoutEditViewRenderer.SELECTED_COMPONENT_BORDER_STYLE);
    }

    private static MACHINE_STATUS_DOT_COLOR = "black";
    private static SELECTED_COMPONENT_BORDER_STYLE = {strokeColor: "red", lineWidth: 1};

    //// Canvas Action Handlers

    public handleMouseMove(e: any): void {
        if (e.touches) e.preventDefault();
        const point = this.eventInputPosition(e);
        const c = this.selectedComponent;
        if (c && (this.state === 'mouse_down' || this.state === 'dragging')) {
            if (this.state === 'mouse_down') {
                if (point.x > c.x && point.x < c.x + c.width &&
                    point.y > c.y && point.y < c.y + c.height) {
                    this.state = 'dragging';
                }
            } else if (this.state === 'dragging') {
                c.x = c.x + (point.x - this.lastMousePosition.x);
                c.y = c.y + (point.y - this.lastMousePosition.y);
                this.lastMousePosition = point;
            }
        } else if (c && this.state === 'resizing') {
            const x = c.x + c.width;
            const xOffset = point.x - x;
            c.width += xOffset;
            c.height += xOffset / c.aspect_ratio;
        }
    }

    public handleMouseUp(e: any): void {
        if (this.state === 'saving') return;
        if (this.state === 'mouse_down') {
            const point = this.eventInputPosition(e);
            const comp = this.layout.current_ordered_component.reverse().find(c => {
                return point.x > c.x && point.x < c.x + c.width && point.y > c.y && point.y < c.y + c.height;
            });
            if (comp) {
                this.setSelectedComponent(comp);
            } else {
                this.setSelectedComponent(undefined);
            }
        }
        this.state = 'done';
        this.lastMousePosition = CanvasRenderer.MOUSE_INITIAL_POSITION;
    }

    public handleMouseDown(e: any): void {
        if (this.state === 'saving') return;
        this.state = 'mouse_down';
        const point = this.eventInputPosition(e);
        this.lastMousePosition = point;
        const c = this.selectedComponent;
        if (c) {
            if (c.pointOverlapsComponentCorner(point)) {
                this.state = 'resizing';
            }
        }
    }

    public handleMouseLeave(): void {
        if (this.state === 'saving') return;
        this.state = 'done';
    }

    public handleKeyDown(e: any): void {
        if (e.keyCode === 46) {
            this.deleteSelectedComponent();
        }
    }
}
